﻿#include <winsock2.h>
#pragma comment(lib, "Ws2_32.lib")
#include <stdio.h>
#include <locale.h>
#include <Ws2tcpip.h>
#include <iostream>
#include <string>
#include <iomanip>

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Function: PowerByLAN
            LAN電源への接続と調光のプログラムサンプル

  Summary:  PowerByLAN Summary.
            LAN電源の通信プロトコル：
              ・TCPプロトコルを利用
              ・LAN電源はサーバーとして動いており、ユーザー側のプログラムの要求を受付けて処理する
              ・ユーザー側のプログラムはクライアントとして動く

            LAN電源の制御プロトコル（以下「LAN電源コマンド」と呼ぶ）：
            ****LAN電源送信コマンド****
            * 6バイト～22バイト
            * 1バイト目：ヘッダー、16進数固定値「7E」
            * 2バイト目：ヘッダー、16進数固定値「5A」
            * 3バイト目：LAN電源のチャンネル
            * 4バイト目以後：LAN電源の調光値、00~FF（0～255）
            *                アドレス
            *                調光時間
            * 後ろから2バイト目：フッター、16進数固定値「0D」
            * 後ろから1バイト目：フッター、16進数固定値「0A」
            ****LAN電源受信コマンド****
            * 6バイト～22バイト
            * 1バイト目：ヘッダー、16進数固定値「7E」
            * 2バイト目：ヘッダー、16進数固定値「5A」
            * 3バイト目：結果コード
            * 4バイト目：LAN電源のチャンネル
            * 5バイト目以後：LAN電源の調光値、00~FF（0～255）
            *                アドレス
            *                調光時間
            * 後ろから2バイト目：フッター、16進数固定値「0D」
            * 後ろから1バイト目：フッター、16進数固定値「0A」
            * 
            ****詳細について、取扱説明書を参照すること****

  Args:     Argument1: IP Address
              引数1：LAN電源のIPアドレス
            Argument2: Port
              引数2：LAN電源のポート番号
            Argument3: Command
              引数3：LAN電源の送信コマンド、長さは6バイト～22バイト
                     コマンドの長さによって、引数の数が変わる
            Argument4: 
              引数4：
              ～
            Argument24:
              引数24：

  Returns:  If successed return 0.
              成功の場合、0を返す。
            If failed return 1.
              失敗の場合、1を返す。

  Created:  2024/8/1

  Version:  2.0

  Copyright © 株式会社オプター Opter Inc.
-----------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    WSADATA wsaData;
    struct sockaddr_in serverAddr;
    SOCKET sock;
    char* serverIP;
    unsigned short serverPort;
    int result;
    char revBuff[64] = { 0 };
    char cmdSend[64] = { 0 };
    int revLen = 0;
    int compare1 = 0;
    int compare2 = 0;
    // 調光コマンドFooter
    const char cmdFooter[2] = { (char)0x0D,(char)0x0A };

    // コマンドプロンプトに日本語を出力するため、システム規定のロケールに設定
    setlocale(LC_ALL, "");

    // 引数の数をチェック
    if (argc < 8) {
        wprintf(L"\nPlease check the number of the arguments. 引数の数を確認してください。\n");
        return 1;
    }

    // 引数1をIPアドレスにセット
    serverIP = argv[1];
    // 引数2をポート番号にセット
    serverPort = atoi(argv[2]);

    // 引数3以後のものを送信コマンドにセット
    for (int i = 3; i < argc; i++) {
        char temp[2] = { 0 };
        memcpy(&temp[0], argv[i], 2);
        // 文字列から16進数のCharに変換
        cmdSend[i-3] = std::stoi(&temp[0], 0, 16);
    }

    // 初期化Initialize Winsock2
    result = WSAStartup(MAKEWORD(2, 2), &wsaData);
    if (result != NO_ERROR) {
        wprintf(L"\nWSAStartup failed with error: %d\n", result);
        return 1;
    }

    // ソケット作成Create a socket for sending data
    sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (sock == INVALID_SOCKET) {
        wprintf(L"\nSocket failed with error: %ld\n", WSAGetLastError());
        WSACleanup();
        return 1;
    }

    // 接続先指定用構造体の準備
    memset(&serverAddr, 0, sizeof(SOCKADDR_IN));
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_port = htons(serverPort);
    inet_pton(AF_INET, serverIP, &serverAddr.sin_addr);

    // LAN電源に接続
    if (connect(sock, (struct sockaddr*)&serverAddr, sizeof(serverAddr)) == SOCKET_ERROR) {
        wprintf(L"\nConnected failed with error:%ld. LAN電源への接続が失敗しました。\n", WSAGetLastError());
    }
    else {
        // コマンドを送信する
        result = send(sock, cmdSend, sizeof(cmdSend), 0);

        // 送信コマンドを表示する
        std::cout << "Send Command / 送信コマンド： " << std::ends;
        for (int i = 0; i < argc - 3; i++) {
            std::cout << std::hex << std::uppercase << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(cmdSend[i])) << " " << std::ends;
        }
        std::cout << "\n" << std::endl;

        // コマンドを受信する
        result = ::recv(sock, revBuff, sizeof(revBuff), 0);
        if (result == -1) return 0; // 有効な通信ではないときに中止する

        // 「命令」による受信コマンドの長さを決める
        switch (revBuff[2]) {
        case 00://エラーの時
            revLen = 6;
            break;
        case 01://調光値読取
            revLen = 8;
            break;
        case 02://調光値出力
            revLen = 7;
            break;
        case 03://マニュアル調光値読取
            revLen = 8;
            break;
        case 04://メモリ読取
            revLen = 9;
            break;
        case 05://メモリ記憶
            revLen = 8;
            break;
        case 06://全メモリ一括読取
            revLen = 23;
            break;
        case 07://全メモリ一括記憶
            revLen = 7;
            break;
        case 0x08://ストロポ調光値読取
            revLen = 10;
            break;
        case 0x09://ストロポ調光
            revLen = 7;
            break;
        case 0x0A://LED点灯
            revLen = 7;
            break;
        case 0x0B://LED消灯
            revLen = 7;
            break;
        case 0x0C://調光値初期化
            revLen = 7;
            break;
        case 0x0D://調光値記憶
            revLen = 7;
            break;
        case 0x0E://調光値出力と記憶
            revLen = 7;
            break;
        default:
            break;
        }

        // 受信コマンドを表示する
        std::cout << "Receive Command / 受信コマンド： " << std::ends;
        for (int i = 0; i < revLen; i++) {
            std::cout << std::hex << std::uppercase << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(revBuff[i])) << " " << std::ends;
        }
        std::cout << "\n" << std::endl;

        // 結果コードを読取り、結果メッセージまたはエラーメッセージを表示する
        switch (revBuff[3]) {
        case 00:
            // 送信コマンドと戻された受信コマンドを比較し、コマンドのヘッダーと種別を表す前の3バイトとコマンドフッターの2バイトが一致しなかったら、エラーを表示する。
            compare1 = memcmp(&cmdSend[0], &revBuff[0], 3);
            compare2 = memcmp(cmdFooter, &revBuff[revLen-2], 2);
            if (compare1 != 0 || compare2 != 0) {
                wprintf(L"\n受信コマンドが送信コマンドと一致していません。");
            }
            else {
                wprintf(L"\n送信コマンドを正しく実行しました。");
            }
            break;
        case 01:
            wprintf(L"\n送信コマンドのヘッダーが検出されませんでした。");
            break;
        case 02:
            wprintf(L"\n送信コマンドのフッターが検出されませんでした。");
            break;
        case 03:
            wprintf(L"\n送信コマンドの命令が正しくありません。");
            break;
        case 04:
            wprintf(L"\n送信コマンドのチャンネルが範囲外です。");
            break;
        case 05:
            wprintf(L"\n送信コマンドのアドレスが範囲外です。");
            break;
        case 06:
            wprintf(L"\n送信コマンドの形式が正しくありません。");
            break;
        case 07:
            wprintf(L"\n電源本体のつまみによる内部調光を使用中のため、外部調光方式が利用できません。");
            break;
        default:
            wprintf(L"\n予想外のエラーを受信しました。");
            break;
        }
    }

    // TCPセッションの終了
    closesocket(sock);

    // winsock2の終了処理
    WSACleanup();

    return 0;
}

